/*
  Program....: JDBCDisplayQueryJDK11.java

*/

import java.sql.*;
import java.security.*;
import java.applet.Applet;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

public class JDBCDisplayQueryJDK11 extends Applet {
   // java.sql Objects
   private Connection connection;
   private Statement statement;
   private ResultSet resultSet;
   private ResultSetMetaData rsMetaData;

   private TextArea inputQuery;
   private Button submitQuery;
   private TextArea txtareaStatus = new TextArea( "Status messages... \n", 4, 30, TextArea.SCROLLBARS_BOTH  );
    
   private boolean blnSuccess = false;

   public void init() 
   {   
      //Tested with Microsoft Access Database using ODBC and User DSN
      String url = "jdbc:odbc:JDBCDisplayQuery_access2000";  
      String username = "guest";
      String password = "guest";
      String [] astrDrivers = {"sun.jdbc.odbc.JdbcOdbcDriver"};

      try {
        
        for ( int i=0; i<astrDrivers.length; i++ ) {
          txtareaStatus.append( "Trying to locate: " + astrDrivers[i] + "\n");        
          Class.forName( astrDrivers[i] ).newInstance();
        }
        
         txtareaStatus.append( "Connecting to: " + url + ", " + username + "/" + password + "\n");
         connection = DriverManager.getConnection( 
            url, username, password );
         txtareaStatus.append( "Connection successful.\n");
         blnSuccess = true;
      } 
      /*
      catch ( AccessControlException e ) {  //Not needed in JDK 1.1 - available to JDK 1.2
         txtareaStatus.append( "Failure: Access Control Exception occured with driver.\n" );        
         System.err.println( "Failure: Access Control Exception occured with driver." );
         e.printStackTrace();
         blnSuccess = false;
      }
      */
      catch ( IllegalAccessException e ) {
         txtareaStatus.append( "Failure: Access denied to JDBC/ODBC driver.\n" );        
         System.err.println( "Failure: Access denied to JDBC/ODBC driver." );
         e.printStackTrace();
         blnSuccess = false;
      }
      catch ( InstantiationException e ) {
         txtareaStatus.append( "Failure: Could not instantiate new JDBC/ODBC driver.\n" );        
         System.err.println( "Failure: Could not instantiate new JDBC/ODBC driver." );
         e.printStackTrace();
         blnSuccess = false;
      }      
      catch ( ClassNotFoundException cnfex ) {
         txtareaStatus.append( "Failure: Cannot locate driver.\n" );        
         System.err.println( "Failure: Cannot locate driver." );
         cnfex.printStackTrace();
         blnSuccess = false;
      }
      catch ( SQLException sqlex ) {
         txtareaStatus.append( "Failure: Unable to connect to DB.\n" );                
         System.err.println( "Failure: Unable to connect to DB" );
         sqlex.printStackTrace();
         blnSuccess = false;
      }

      catch ( Exception e ) {
      	 String strErrMsg = "Failure: Exception caught.\n" + 
                            "toString() = " + e.toString() + "\n" +
                            "getMessage() = " + e.getMessage() + "\n";
         txtareaStatus.append( strErrMsg );

         System.err.println( "Failure: Exception caught.\n" + strErrMsg );
         System.out.println( "Failure: Exception caught.\n" + strErrMsg );
         e.printStackTrace();
         blnSuccess = false;
        
      }

      inputQuery = new TextArea( "SELECT * FROM Test", 4, 30, TextArea.SCROLLBARS_BOTH  );
      submitQuery = new Button( "Submit query" );
      submitQuery.addActionListener(
         new ActionListener() {
            public void actionPerformed( ActionEvent e ) 
            {
              if ( blnSuccess ) {
                getTable();
              }
            }
         }   
      );

      Panel topPanel = new Panel();
      topPanel.setLayout( new BorderLayout() );
      
      topPanel.add( inputQuery, BorderLayout.CENTER );
      Panel pnlButton = new Panel();
      pnlButton.add( submitQuery );
      topPanel.add( pnlButton, BorderLayout.SOUTH );
 
      this.setLayout( new BorderLayout() );
      this.add( topPanel, BorderLayout.NORTH );
      this.add( (new ScrollPane()), BorderLayout.CENTER );      //Adds a blank ScrollPane
      this.add( txtareaStatus, BorderLayout.SOUTH );
      
      if ( blnSuccess ) {
        getTable();
      } else {
        //System.exit( 1 );  //Could terminate program here, but not user friendly.
      }

      setSize( 500, 500 );
      setVisible( true );
   }

   private void getTable()
   {
      try {
         String query = inputQuery.getText();
         txtareaStatus.append( "Query string: " + query );
         statement = connection.createStatement();
         resultSet = statement.executeQuery( query );         
         displayResultSet( resultSet );
      }
      catch ( SQLException sqlex ) {
         sqlex.printStackTrace();
      }
   }

   private void displayResultSet( ResultSet rs )
      throws SQLException
   {
      // Set cursor to first record.
      boolean moreRecords = rs.next();  

      // IF no records.
      if ( ! moreRecords ) {
        txtareaStatus.append( "No records found.\n" );
        System.out.println("No records found.");
         return;
      }

      Vector columnHeads = new Vector();
      Vector rows = new Vector();

      try {
         // get column heads
         ResultSetMetaData rsmd = rs.getMetaData();
      
         for ( int i = 1; i <= rsmd.getColumnCount(); ++i ) 
            columnHeads.addElement( rsmd.getColumnName( i ) );

         // get row data
         do {
            rows.addElement( getNextRow( rs, rsmd ) ); 
         } while ( rs.next() );

         // display table with ResultSet contents
         
         ScrollPane awtscroller = new CustAwtTable( rows, columnHeads ); //awt added.

         this.remove( 1 );
         this.add( awtscroller, BorderLayout.CENTER );
         this.validate();
      }
      catch ( SQLException sqlex ) {
         sqlex.printStackTrace();
      }
   }
   
   private Vector getNextRow( ResultSet rs, 
                              ResultSetMetaData rsmd )
       throws SQLException
   {
      Vector currentRow = new Vector();
      
      for ( int i = 1; i <= rsmd.getColumnCount(); ++i )
         switch( rsmd.getColumnType( i ) ) {
            case Types.VARCHAR:
            case Types.LONGVARCHAR:
                  currentRow.addElement( rs.getString( i ) );
               break;
            case Types.INTEGER:
                  currentRow.addElement( 
                     new Long( rs.getLong( i ) ) );
               break;
            default: 
               System.out.println( "Type was: " + 
                  rsmd.getColumnTypeName( i ) );
         }
      
      return currentRow;
   }

   public void shutDown()
   {
      try {
         connection.close();
      }
      catch ( SQLException sqlex ) {
         System.err.println( "Unable to disconnect" );
         sqlex.printStackTrace();
      }
   }
  public static void main( String args [] )
  { 
    Frame app = new Frame( "JDBCDisplayQueryJDK11.java" );  
    app.setSize( 500, 500);                         //set Frame: width, height

    final JDBCDisplayQueryJDK11 applet = new JDBCDisplayQueryJDK11();         //create the applet.
    
    app.addWindowListener( 
         new WindowAdapter() {
            public void windowClosing( WindowEvent e ) 
            {  
               applet.shutDown();
               System.exit( 0 );
            }
         }
    );

    app.add( applet, BorderLayout.CENTER );       //add applet to center of frame.
    applet.init();                                //initialize applet.
    applet.start();                               //start applet.
    app.setVisible( true );                    //Make frame visible.

  } //main()

  class CustAwtTable extends ScrollPane {
    Vector avecCurrentRow;
    
    public CustAwtTable ( Vector avecRows, Vector avecHeadings ) {
      
      int intRow    = avecRows.size();
      int intColumn = avecHeadings.size();
      Label lblText;
      
      Panel pnlTable = new Panel();
      GridBagLayout gblLayout = new GridBagLayout();
      GridBagConstraints gbcConstraints = new GridBagConstraints();
      
      pnlTable.setLayout( gblLayout );
      
      gbcConstraints.weightx = 1.0;
      gbcConstraints.weighty = 0.0;
      gbcConstraints.gridx = GridBagConstraints.RELATIVE;  //Default is RELATIVE
      gbcConstraints.gridy = GridBagConstraints.RELATIVE;  //Default is RELATIVE
      gbcConstraints.gridwidth = 1;
      gbcConstraints.gridheight = 1;
      gbcConstraints.anchor = GridBagConstraints.NORTHWEST;
            
      for ( int x = 0; x < intColumn; x++ ) {
        lblText = new Label ( (String) avecHeadings.elementAt(x) );
        if ( x == intColumn -1 ) {
          gbcConstraints.gridwidth = GridBagConstraints.REMAINDER ; //End of Row
        }
        gblLayout.setConstraints( lblText, gbcConstraints);
        pnlTable.add( lblText );
      }
      
      for ( int x = 0; x < intRow; x++ ) {
        gbcConstraints.gridwidth = 1;  //Reset the gridwidth        
        avecCurrentRow = (Vector) avecRows.elementAt(x);
        int intCurrentSize = avecCurrentRow.size();
        for ( int intCol = 0; intCol < intCurrentSize ; intCol++ ) {
          lblText = new Label ( (String) ""+avecCurrentRow.elementAt(intCol) );
          if ( intCol == intCurrentSize -1 ) {
            gbcConstraints.gridwidth = GridBagConstraints.REMAINDER ; //End of Row
          }
          if ( x == intRow -1 ) {
            gbcConstraints.gridheight = GridBagConstraints.REMAINDER ; //End of Table
          }
          gblLayout.setConstraints( lblText, gbcConstraints);          
          pnlTable.add( lblText );
        }
      }      
      
      this.add( pnlTable );
    }
    
  }

}